rm( list=ls() )   
set.seed(1)  
source("functions_solution.R")

p=2
fct_code = function(x,beta) {
  #x: vector of n experimental conditions
  #beta: vector of two model parameters
  #RETURN: vector of code responses
  beta[1] + beta[2] * x
}

fct_H = function(x,beta) {
  #x: vector of n experimental conditions
  #beta: vector of two model parameters
  #RETURN: matrix of partial derivatives at beta (cf slides)
  cbind(1,x)
}

fct_physical_system = function(x) {
  #x: vector of n experimental conditions
  #RETURN: vector of code responses
  -sin(pi*x/1.3)
}

sigma_m = 0.1

#Data
n=50
X = (1:n)/n   #vector of experimental conditions
n=length(X)
y = fct_physical_system(X) + rnorm(n=n,mean=0,sd=sigma_m)  #vector of experimental results
beta_nom = c(1,1)  #
f_beta_nom = fct_code(X,beta_nom)
H = fct_H(X,beta_nom)

#A priori on beta
beta_prior = c(-0.6,0)
Q_prior = diag(c(0.36,0.36))

#Plot of the a priori parameters
X_cont = (1:100)/100
plot(X,y,type="p",col="black")
points(X_cont,fct_code(X_cont,beta_prior),type="l",col="blue")
points(X_cont,fct_physical_system(X_cont),type="l",col="black")

#Estimating the measure error by restricted maximum likelihood
hat_sigma_m = sqrt( (1/(n-p)) * t(y) %*% (diag(n) - H%*% solve(t(H)%*%H)%*%t(H))%*%y )
hat_sigma_m = as.numeric(hat_sigma_m)

#Computing a posteriori beta
R = hat_sigma_m^2 * diag(n)
beta_post = fct_beta_post(beta_prior,Q_prior,H,R,y,f_beta_nom,beta_nom)

#Adding the a posteriori parameters
points(X_cont,fct_code(X_cont,beta_post),type="l",col="red")

#computing and plotting the 95 percent confidence intervals
Q_post=fct_Q_post(Q_prior,H,R)
H_cont = fct_H(X_cont,beta_nom)
cond_var_cont = diag(H_cont%*%Q_post%*%t(H_cont))  #vector of the conditional variances of
                                                   #the physical system at the points in
points(X_cont,fct_code(X_cont,beta_post)+1.96*sqrt(cond_var_cont),type="l",col="green")
points(X_cont,fct_code(X_cont,beta_post)-1.96*sqrt(cond_var_cont),type="l",col="green")













