rm( list=ls() )   
set.seed(2)  
source("functions.R")

p=3
d=3
fct_code = function(x,beta) {
  #x: n*3 matrix of n experimental conditions
  #beta: vector of two model parameters
  #RETURN: vector of code responses
  beta[1]*x[,1] + beta[2]^2*x[,2]^2 + (1+x[,3])^(0.1+beta[3]^2)
}

fct_H = function(x,beta) {
  #x: n*3 matrix of n experimental conditions
  #beta: vector of two model parameters
  #RETURN: matrix of partial derivatives at beta (cf slide 10)
  cbind(x[,1],2*beta[2]*x[,2]^2,(1+x[,3])^(0.1+beta[3]^2)*2*beta[3]*log(1+x[,3]))
}

fct_physical_system = function(x) {
  #x: vector of n experimental conditions
  #RETURN: vector of code responses
  x[,1] + x[,2]^2 + (1+x[,3])^(0.1+0.6^2) + 1.5*(1+x[,1])*(0.3*x[,2]) -1.5/(1+x[,3]^2)
}

sigma_m = 0.1

#Data
n=50
X = matrix(nrow=n,ncol=d,data=runif(n*d))   #matrix of n experimental conditions

y = #TO COMPLETE  #vector of experimental results
beta_nom = c(1,1,0.6)  
f_beta_nom = fct_code(X,beta_nom)
H_nom = fct_H(X,beta_nom)

#A priori on beta
beta_prior = beta_nom  
Q_prior = diag(c(0.5,0.5,0.5))

#covariance parameters (fixed)
sigma=1
l = c(0.6,0.6,0.6)
R = fct_cov_matern_3_2(X,X,sigma,l)+sigma_m^2*diag(n)

#Prediction performance of the a priori code
n_cont=500
X_cont = matrix(nrow=n_cont,ncol=d,data=runif(n_cont*d))
y_cont = fct_physical_system(X_cont)
pred_code_prior = fct_code(X_cont,beta_prior)
cat("RMSE calibrated code a priori: ",sqrt(mean((pred_code_prior-y_cont)^2)),"\n")

#Iterative calibration of the code
n_iterative_cal = 5
m_beta_post = matrix(nrow=n_iterative_cal,ncol=d,data=0) #matrix of the
                            #iterated beta_post, cf slide  72
m_beta_post[1,] = beta_nom
cat("beta_post 1 = beta_nom:",beta_nom,"\n")
for (i in 2:n_iterative_cal)  {    #cf loop slide 72
  #TO COMPLETE
  beta_post_i = #TO COMPLETE
  pred_code_i = fct_code(X_cont,beta_post_i)
  cat("beta_post ",i,":",beta_post_i,"\n")
  cat("RMSE calibrated code",i,":",sqrt(mean((pred_code_i-y_cont)^2)),"\n")
}


#Computing conditional means and variances at points in X_cont for all 
                  #beta_post_i of the iterative calibration
                  #cf slide 73
for (i in 2:n_iterative_cal) {    
  hat_phi_cont_i = 1:n_cont    # Will contain the n_cont 
  #          predictions of the physical for input values in X_cont
  hat_sigma_2_cont_i = 1:n_cont # Will contain the n_cont 
  #          conditional variances of the physical for input values in X_cont
  beta_nom_i = m_beta_post[i-1,]
  beta_post_i = m_beta_post[i,]
  H_i = fct_H(X,beta_nom_i)
  f_nom_i = fct_code(X,beta_nom_i)
  for (j in 1:n_cont) {
    #TO COMPLETE
    hat_phi_cont_i[j] = # TO COMPLETE
    hat_sigma_2_cont_i[j] = #TO COMPLETE
  }
  cat("RMSE gaussian process prediction with beta nominal",i,": ",
      sqrt(mean((hat_phi_cont_i-y_cont)^2)),"\n")
}


#Computing Qpost with the linearization for the beta_nom obtained at the
    #end of the iterative calibration
beta_nom_iter = m_beta_post[n_iterative_cal,]
H_iter = fct_H(X,beta_nom_iter)
Q_post_iter = fct_Q_post(Q_prior,H_iter,R)
cat("diagonal components of Qpost after iterative linearization: ",
    diag(Q_post_iter),"\n")

#Computing and storing the pdf and confitional pdf  #cf lecture slides74, 75
N=10000
m_beta_int = matrix(nrow=N,ncol=d,data=runif(min=-1,max=2,n=N*d))
v_log_p_y_given_beta = 1:N   #Will contain the p(y | beta_i) in Problem 2 description
v_log_p_beta = 1:N    #Will contain the p(beta_i) in Problem 2 description
for (i in 1:N) {   #loop on beta_1,...,beta_N
  beta_i = m_beta_int[i,]
  v_log_p_y_given_beta[i] = #TO COMPLETE 
    v_log_p_beta[i] = #TO COMPLETE
}
w_p_y_given_beta = #TO COMPLETE #contain the bar(p)(y | beta_i) in Problem 2 description
w_p_beta = #TO COMPLETE #contain the bar(p)( beta_i) in Problem 2 description


#Computing non-linear conditional means and conditional
#        variance of beta cf lecture slide 75
cond_mean_beta = #TO COMPLETE
cat("non-linear conditional mean beta: ",cond_mean_beta,"\n")

cond_var_beta = #TO COMPLETE
cat("non-linear conditional variance beta: ",cond_var_beta,"\n")

#Computing non-linear conditional means of physical system, cf slide 77
m_hat_phi = matrix(nrow=N,ncol=n_cont,data=0)   #Will contain all
#             the E(phi(x_0)|y,beta_i), cf slide 76, 77
for (i in 1:N) {   #Computing the N*n_cont matrix of predictions
  #         for the N beta, cf slide 76,77
  beta_i = m_beta_int[i,]
  m_hat_phi[i,] = #TO COMPLETE
}
hat_phi_non_linear = # TO COMPLETE #The N conditional means
  #of phi(x_0), cf slide 77 
cat("RMSE non_linear conditional mean of physical system: ",
    sqrt(mean((hat_phi_non_linear-y_cont)^2)),"\n")


